@@ -36,4 +36,5 @@ group by
     s_name
 order by
     numwait desc,
-    s_name;
\ No newline at end of file
+    s_name
+limit 100;
