@@ -32,7 +32,7 @@ use tokio::runtime::Runtime;
 fn create_context(field_count: u32) -> datafusion_common::Result<Arc<SessionContext>> {
     let mut fields = vec![];
     for i in 0..field_count {
-        fields.push(Field::new(format!("str{}", i), DataType::Utf8, true))
+        fields.push(Field::new(format!("str{i}"), DataType::Utf8, true))
     }
 
     let schema = Arc::new(Schema::new(fields));
@@ -49,8 +49,8 @@ fn run(column_count: u32, ctx: Arc<SessionContext>, rt: &Runtime) {
         let mut data_frame = ctx.table("t").await.unwrap();
 
         for i in 0..column_count {
-            let field_name = &format!("str{}", i);
-            let new_field_name = &format!("newstr{}", i);
+            let field_name = &format!("str{i}");
+            let new_field_name = &format!("newstr{i}");
 
             data_frame = data_frame
                 .with_column_renamed(field_name, new_field_name)
