@@ -20,6 +20,7 @@ mod distinct_count_string_fuzz;
 mod join_fuzz;
 mod merge_fuzz;
 mod sort_fuzz;
+mod sort_query_fuzz;
 
 mod aggregation_fuzzer;
 mod equivalence;
@@ -29,3 +30,6 @@ mod pruning;
 mod limit_fuzz;
 mod sort_preserving_repartition_fuzz;
 mod window_fuzz;
+
+// Utility modules
+mod record_batch_generator;
