@@ -17,13 +17,15 @@
 
 use std::sync::Arc;
 
-use arrow::array::{ArrayRef, RecordBatch};
+use arrow::array::{ArrayRef, DictionaryArray, PrimitiveArray, RecordBatch};
 use arrow::datatypes::{
-    BooleanType, DataType, Date32Type, Date64Type, Decimal128Type, Decimal256Type, Field,
-    Float32Type, Float64Type, Int16Type, Int32Type, Int64Type, Int8Type,
-    IntervalDayTimeType, IntervalMonthDayNanoType, IntervalUnit, IntervalYearMonthType,
-    Schema, Time32MillisecondType, Time32SecondType, Time64MicrosecondType,
-    Time64NanosecondType, TimeUnit, TimestampMicrosecondType, TimestampMillisecondType,
+    ArrowPrimitiveType, BooleanType, DataType, Date32Type, Date64Type, Decimal128Type,
+    Decimal256Type, DurationMicrosecondType, DurationMillisecondType,
+    DurationNanosecondType, DurationSecondType, Field, Float32Type, Float64Type,
+    Int16Type, Int32Type, Int64Type, Int8Type, IntervalDayTimeType,
+    IntervalMonthDayNanoType, IntervalUnit, IntervalYearMonthType, Schema,
+    Time32MillisecondType, Time32SecondType, Time64MicrosecondType, Time64NanosecondType,
+    TimeUnit, TimestampMicrosecondType, TimestampMillisecondType,
     TimestampNanosecondType, TimestampSecondType, UInt16Type, UInt32Type, UInt64Type,
     UInt8Type,
 };
@@ -32,7 +34,7 @@ use arrow_schema::{
     DECIMAL256_MAX_SCALE,
 };
 use datafusion_common::{arrow_datafusion_err, DataFusionError, Result};
-use rand::{rngs::StdRng, thread_rng, Rng, SeedableRng};
+use rand::{rng, rngs::StdRng, Rng, SeedableRng};
 use test_utils::array_gen::{
     BinaryArrayGenerator, BooleanArrayGenerator, DecimalArrayGenerator,
     PrimitiveArrayGenerator, StringArrayGenerator,
@@ -85,16 +87,33 @@ pub fn get_supported_types_columns(rng_seed: u64) -> Vec<ColumnDescr> {
             "interval_month_day_nano",
             DataType::Interval(IntervalUnit::MonthDayNano),
         ),
+        // Internal error: AggregationFuzzer task error: JoinError::Panic(Id(29108), "called `Option::unwrap()` on a `None` value", ...).
+        // ColumnDescr::new(
+        //     "duration_seconds",
+        //     DataType::Duration(TimeUnit::Second),
+        // ),
+        ColumnDescr::new(
+            "duration_milliseconds",
+            DataType::Duration(TimeUnit::Millisecond),
+        ),
+        ColumnDescr::new(
+            "duration_microsecond",
+            DataType::Duration(TimeUnit::Microsecond),
+        ),
+        ColumnDescr::new(
+            "duration_nanosecond",
+            DataType::Duration(TimeUnit::Nanosecond),
+        ),
         ColumnDescr::new("decimal128", {
-            let precision: u8 = rng.gen_range(1..=DECIMAL128_MAX_PRECISION);
-            let scale: i8 = rng.gen_range(
+            let precision: u8 = rng.random_range(1..=DECIMAL128_MAX_PRECISION);
+            let scale: i8 = rng.random_range(
                 i8::MIN..=std::cmp::min(precision as i8, DECIMAL128_MAX_SCALE),
             );
             DataType::Decimal128(precision, scale)
         }),
         ColumnDescr::new("decimal256", {
-            let precision: u8 = rng.gen_range(1..=DECIMAL256_MAX_PRECISION);
-            let scale: i8 = rng.gen_range(
+            let precision: u8 = rng.random_range(1..=DECIMAL256_MAX_PRECISION);
+            let scale: i8 = rng.random_range(
                 i8::MIN..=std::cmp::min(precision as i8, DECIMAL256_MAX_SCALE),
             );
             DataType::Decimal256(precision, scale)
@@ -108,6 +127,11 @@ pub fn get_supported_types_columns(rng_seed: u64) -> Vec<ColumnDescr> {
         ColumnDescr::new("binary", DataType::Binary),
         ColumnDescr::new("large_binary", DataType::LargeBinary),
         ColumnDescr::new("binaryview", DataType::BinaryView),
+        ColumnDescr::new(
+            "dictionary_utf8_low",
+            DataType::Dictionary(Box::new(DataType::UInt64), Box::new(DataType::Utf8)),
+        )
+        .with_max_num_distinct(10),
     ]
 }
 
@@ -161,22 +185,19 @@ pub struct RecordBatchGenerator {
 
     /// If a seed is provided when constructing the generator, it will be used to
     /// create `rng` and the pseudo-randomly generated batches will be deterministic.
-    /// Otherwise, `rng` will be initialized using `thread_rng()` and the batches
+    /// Otherwise, `rng` will be initialized using `rng()` and the batches
     /// generated will be different each time.
     rng: StdRng,
 }
 
 macro_rules! generate_decimal_array {
-    ($SELF:ident, $NUM_ROWS:ident, $MAX_NUM_DISTINCT: expr, $BATCH_GEN_RNG:ident, $ARRAY_GEN_RNG:ident, $PRECISION: ident, $SCALE: ident, $ARROW_TYPE: ident) => {{
-        let null_pct_idx = $BATCH_GEN_RNG.gen_range(0..$SELF.candidate_null_pcts.len());
-        let null_pct = $SELF.candidate_null_pcts[null_pct_idx];
-
+    ($SELF:ident, $NUM_ROWS:ident, $MAX_NUM_DISTINCT: expr, $NULL_PCT:ident, $BATCH_GEN_RNG:ident, $ARRAY_GEN_RNG:ident, $PRECISION: ident, $SCALE: ident, $ARROW_TYPE: ident) => {{
         let mut generator = DecimalArrayGenerator {
             precision: $PRECISION,
             scale: $SCALE,
             num_decimals: $NUM_ROWS,
             num_distinct_decimals: $MAX_NUM_DISTINCT,
-            null_pct,
+            null_pct: $NULL_PCT,
             rng: $ARRAY_GEN_RNG,
         };
 
@@ -186,17 +207,13 @@ macro_rules! generate_decimal_array {
 
 // Generating `BooleanArray` due to it being a special type in Arrow (bit-packed)
 macro_rules! generate_boolean_array {
-    ($SELF:ident, $NUM_ROWS:ident, $MAX_NUM_DISTINCT:expr, $BATCH_GEN_RNG:ident, $ARRAY_GEN_RNG:ident, $ARROW_TYPE: ident) => {{
-        // Select a null percentage from the candidate percentages
-        let null_pct_idx = $BATCH_GEN_RNG.gen_range(0..$SELF.candidate_null_pcts.len());
-        let null_pct = $SELF.candidate_null_pcts[null_pct_idx];
-
+    ($SELF:ident, $NUM_ROWS:ident, $MAX_NUM_DISTINCT:expr, $NULL_PCT:ident, $BATCH_GEN_RNG:ident, $ARRAY_GEN_RNG:ident, $ARROW_TYPE: ident) => {{
         let num_distinct_booleans = if $MAX_NUM_DISTINCT >= 2 { 2 } else { 1 };
 
         let mut generator = BooleanArrayGenerator {
             num_booleans: $NUM_ROWS,
             num_distinct_booleans,
-            null_pct,
+            null_pct: $NULL_PCT,
             rng: $ARRAY_GEN_RNG,
         };
 
@@ -205,14 +222,11 @@ macro_rules! generate_boolean_array {
 }
 
 macro_rules! generate_primitive_array {
-    ($SELF:ident, $NUM_ROWS:ident, $MAX_NUM_DISTINCT:expr, $BATCH_GEN_RNG:ident, $ARRAY_GEN_RNG:ident, $ARROW_TYPE:ident) => {{
-        let null_pct_idx = $BATCH_GEN_RNG.gen_range(0..$SELF.candidate_null_pcts.len());
-        let null_pct = $SELF.candidate_null_pcts[null_pct_idx];
-
+    ($SELF:ident, $NUM_ROWS:ident, $MAX_NUM_DISTINCT:expr, $NULL_PCT:ident, $BATCH_GEN_RNG:ident, $ARRAY_GEN_RNG:ident, $ARROW_TYPE:ident) => {{
         let mut generator = PrimitiveArrayGenerator {
             num_primitives: $NUM_ROWS,
             num_distinct_primitives: $MAX_NUM_DISTINCT,
-            null_pct,
+            null_pct: $NULL_PCT,
             rng: $ARRAY_GEN_RNG,
         };
 
@@ -220,6 +234,28 @@ macro_rules! generate_primitive_array {
     }};
 }
 
+macro_rules! generate_dict {
+    ($SELF:ident, $NUM_ROWS:ident, $MAX_NUM_DISTINCT:expr, $NULL_PCT:ident, $BATCH_GEN_RNG:ident, $ARRAY_GEN_RNG:ident, $ARROW_TYPE:ident, $VALUES: ident) => {{
+        debug_assert_eq!($VALUES.len(), $MAX_NUM_DISTINCT);
+        let keys: PrimitiveArray<$ARROW_TYPE> = (0..$NUM_ROWS)
+            .map(|_| {
+                if $BATCH_GEN_RNG.random::<f64>() < $NULL_PCT {
+                    None
+                } else if $MAX_NUM_DISTINCT > 1 {
+                    let range = 0..($MAX_NUM_DISTINCT
+                        as <$ARROW_TYPE as ArrowPrimitiveType>::Native);
+                    Some($ARRAY_GEN_RNG.random_range(range))
+                } else {
+                    Some(0)
+                }
+            })
+            .collect();
+
+        let dict = DictionaryArray::new(keys, $VALUES);
+        Arc::new(dict) as ArrayRef
+    }};
+}
+
 impl RecordBatchGenerator {
     /// Create a new `RecordBatchGenerator` with a random seed. The generated
     /// batches will be different each time.
@@ -235,7 +271,7 @@ impl RecordBatchGenerator {
             max_rows_num,
             columns,
             candidate_null_pcts,
-            rng: StdRng::from_rng(thread_rng()).unwrap(),
+            rng: StdRng::from_rng(&mut rng()),
         }
     }
 
@@ -247,9 +283,9 @@ impl RecordBatchGenerator {
     }
 
     pub fn generate(&mut self) -> Result<RecordBatch> {
-        let num_rows = self.rng.gen_range(self.min_rows_num..=self.max_rows_num);
-        let array_gen_rng = StdRng::from_seed(self.rng.gen());
-        let mut batch_gen_rng = StdRng::from_seed(self.rng.gen());
+        let num_rows = self.rng.random_range(self.min_rows_num..=self.max_rows_num);
+        let array_gen_rng = StdRng::from_seed(self.rng.random());
+        let mut batch_gen_rng = StdRng::from_seed(self.rng.random());
         let columns = self.columns.clone();
 
         // Build arrays
@@ -281,9 +317,28 @@ impl RecordBatchGenerator {
         num_rows: usize,
         batch_gen_rng: &mut StdRng,
         array_gen_rng: StdRng,
+    ) -> ArrayRef {
+        let null_pct_idx = batch_gen_rng.random_range(0..self.candidate_null_pcts.len());
+        let null_pct = self.candidate_null_pcts[null_pct_idx];
+
+        Self::generate_array_of_type_inner(
+            col,
+            num_rows,
+            batch_gen_rng,
+            array_gen_rng,
+            null_pct,
+        )
+    }
+
+    fn generate_array_of_type_inner(
+        col: &ColumnDescr,
+        num_rows: usize,
+        batch_gen_rng: &mut StdRng,
+        array_gen_rng: StdRng,
+        null_pct: f64,
     ) -> ArrayRef {
         let num_distinct = if num_rows > 1 {
-            batch_gen_rng.gen_range(1..num_rows)
+            batch_gen_rng.random_range(1..num_rows)
         } else {
             num_rows
         };
@@ -299,6 +354,7 @@ impl RecordBatchGenerator {
                     self,
                     num_rows,
                     max_num_distinct,
+                    null_pct,
                     batch_gen_rng,
                     array_gen_rng,
                     Int8Type
@@ -309,6 +365,7 @@ impl RecordBatchGenerator {
                     self,
                     num_rows,
                     max_num_distinct,
+                    null_pct,
                     batch_gen_rng,
                     array_gen_rng,
                     Int16Type
@@ -319,6 +376,7 @@ impl RecordBatchGenerator {
                     self,
                     num_rows,
                     max_num_distinct,
+                    null_pct,
                     batch_gen_rng,
                     array_gen_rng,
                     Int32Type
@@ -329,6 +387,7 @@ impl RecordBatchGenerator {
                     self,
                     num_rows,
                     max_num_distinct,
+                    null_pct,
                     batch_gen_rng,
                     array_gen_rng,
                     Int64Type
@@ -339,6 +398,7 @@ impl RecordBatchGenerator {
                     self,
                     num_rows,
                     max_num_distinct,
+                    null_pct,
                     batch_gen_rng,
                     array_gen_rng,
                     UInt8Type
@@ -349,6 +409,7 @@ impl RecordBatchGenerator {
                     self,
                     num_rows,
                     max_num_distinct,
+                    null_pct,
                     batch_gen_rng,
                     array_gen_rng,
                     UInt16Type
@@ -359,6 +420,7 @@ impl RecordBatchGenerator {
                     self,
                     num_rows,
                     max_num_distinct,
+                    null_pct,
                     batch_gen_rng,
                     array_gen_rng,
                     UInt32Type
@@ -369,6 +431,7 @@ impl RecordBatchGenerator {
                     self,
                     num_rows,
                     max_num_distinct,
+                    null_pct,
                     batch_gen_rng,
                     array_gen_rng,
                     UInt64Type
@@ -379,6 +442,7 @@ impl RecordBatchGenerator {
                     self,
                     num_rows,
                     max_num_distinct,
+                    null_pct,
                     batch_gen_rng,
                     array_gen_rng,
                     Float32Type
@@ -389,6 +453,7 @@ impl RecordBatchGenerator {
                     self,
                     num_rows,
                     max_num_distinct,
+                    null_pct,
                     batch_gen_rng,
                     array_gen_rng,
                     Float64Type
@@ -399,6 +464,7 @@ impl RecordBatchGenerator {
                     self,
                     num_rows,
                     max_num_distinct,
+                    null_pct,
                     batch_gen_rng,
                     array_gen_rng,
                     Date32Type
@@ -409,6 +475,7 @@ impl RecordBatchGenerator {
                     self,
                     num_rows,
                     max_num_distinct,
+                    null_pct,
                     batch_gen_rng,
                     array_gen_rng,
                     Date64Type
@@ -419,6 +486,7 @@ impl RecordBatchGenerator {
                     self,
                     num_rows,
                     max_num_distinct,
+                    null_pct,
                     batch_gen_rng,
                     array_gen_rng,
                     Time32SecondType
@@ -429,6 +497,7 @@ impl RecordBatchGenerator {
                     self,
                     num_rows,
                     max_num_distinct,
+                    null_pct,
                     batch_gen_rng,
                     array_gen_rng,
                     Time32MillisecondType
@@ -439,6 +508,7 @@ impl RecordBatchGenerator {
                     self,
                     num_rows,
                     max_num_distinct,
+                    null_pct,
                     batch_gen_rng,
                     array_gen_rng,
                     Time64MicrosecondType
@@ -449,6 +519,7 @@ impl RecordBatchGenerator {
                     self,
                     num_rows,
                     max_num_distinct,
+                    null_pct,
                     batch_gen_rng,
                     array_gen_rng,
                     Time64NanosecondType
@@ -459,6 +530,7 @@ impl RecordBatchGenerator {
                     self,
                     num_rows,
                     max_num_distinct,
+                    null_pct,
                     batch_gen_rng,
                     array_gen_rng,
                     IntervalYearMonthType
@@ -469,6 +541,7 @@ impl RecordBatchGenerator {
                     self,
                     num_rows,
                     max_num_distinct,
+                    null_pct,
                     batch_gen_rng,
                     array_gen_rng,
                     IntervalDayTimeType
@@ -479,16 +552,62 @@ impl RecordBatchGenerator {
                     self,
                     num_rows,
                     max_num_distinct,
+                    null_pct,
                     batch_gen_rng,
                     array_gen_rng,
                     IntervalMonthDayNanoType
                 )
             }
+            DataType::Duration(TimeUnit::Second) => {
+                generate_primitive_array!(
+                    self,
+                    num_rows,
+                    max_num_distinct,
+                    null_pct,
+                    batch_gen_rng,
+                    array_gen_rng,
+                    DurationSecondType
+                )
+            }
+            DataType::Duration(TimeUnit::Millisecond) => {
+                generate_primitive_array!(
+                    self,
+                    num_rows,
+                    max_num_distinct,
+                    null_pct,
+                    batch_gen_rng,
+                    array_gen_rng,
+                    DurationMillisecondType
+                )
+            }
+            DataType::Duration(TimeUnit::Microsecond) => {
+                generate_primitive_array!(
+                    self,
+                    num_rows,
+                    max_num_distinct,
+                    null_pct,
+                    batch_gen_rng,
+                    array_gen_rng,
+                    DurationMicrosecondType
+                )
+            }
+            DataType::Duration(TimeUnit::Nanosecond) => {
+                generate_primitive_array!(
+                    self,
+                    num_rows,
+                    max_num_distinct,
+                    null_pct,
+                    batch_gen_rng,
+                    array_gen_rng,
+                    DurationNanosecondType
+                )
+            }
             DataType::Timestamp(TimeUnit::Second, None) => {
                 generate_primitive_array!(
                     self,
                     num_rows,
                     max_num_distinct,
+                    null_pct,
                     batch_gen_rng,
                     array_gen_rng,
                     TimestampSecondType
@@ -499,6 +618,7 @@ impl RecordBatchGenerator {
                     self,
                     num_rows,
                     max_num_distinct,
+                    null_pct,
                     batch_gen_rng,
                     array_gen_rng,
                     TimestampMillisecondType
@@ -509,6 +629,7 @@ impl RecordBatchGenerator {
                     self,
                     num_rows,
                     max_num_distinct,
+                    null_pct,
                     batch_gen_rng,
                     array_gen_rng,
                     TimestampMicrosecondType
@@ -519,16 +640,14 @@ impl RecordBatchGenerator {
                     self,
                     num_rows,
                     max_num_distinct,
+                    null_pct,
                     batch_gen_rng,
                     array_gen_rng,
                     TimestampNanosecondType
                 )
             }
             DataType::Utf8 | DataType::LargeUtf8 | DataType::Utf8View => {
-                let null_pct_idx =
-                    batch_gen_rng.gen_range(0..self.candidate_null_pcts.len());
-                let null_pct = self.candidate_null_pcts[null_pct_idx];
-                let max_len = batch_gen_rng.gen_range(1..50);
+                let max_len = batch_gen_rng.random_range(1..50);
 
                 let mut generator = StringArrayGenerator {
                     max_len,
@@ -546,10 +665,7 @@ impl RecordBatchGenerator {
                 }
             }
             DataType::Binary | DataType::LargeBinary | DataType::BinaryView => {
-                let null_pct_idx =
-                    batch_gen_rng.gen_range(0..self.candidate_null_pcts.len());
-                let null_pct = self.candidate_null_pcts[null_pct_idx];
-                let max_len = batch_gen_rng.gen_range(1..100);
+                let max_len = batch_gen_rng.random_range(1..100);
 
                 let mut generator = BinaryArrayGenerator {
                     max_len,
@@ -571,6 +687,7 @@ impl RecordBatchGenerator {
                     self,
                     num_rows,
                     max_num_distinct,
+                    null_pct,
                     batch_gen_rng,
                     array_gen_rng,
                     precision,
@@ -583,6 +700,7 @@ impl RecordBatchGenerator {
                     self,
                     num_rows,
                     max_num_distinct,
+                    null_pct,
                     batch_gen_rng,
                     array_gen_rng,
                     precision,
@@ -595,11 +713,43 @@ impl RecordBatchGenerator {
                     self,
                     num_rows,
                     max_num_distinct,
+                    null_pct,
                     batch_gen_rng,
                     array_gen_rng,
                     BooleanType
                 }
             }
+            DataType::Dictionary(ref key_type, ref value_type)
+                if key_type.is_dictionary_key_type() =>
+            {
+                // We generate just num_distinct values because they will be reused by different keys
+                let mut array_gen_rng = array_gen_rng;
+
+                let values = Self::generate_array_of_type_inner(
+                    &ColumnDescr::new("values", *value_type.clone()),
+                    num_distinct,
+                    batch_gen_rng,
+                    array_gen_rng.clone(),
+                    // Once https://github.com/apache/datafusion/issues/16228 is fixed
+                    // we can also generate nulls in values
+                    0.0, // null values are generated on the key level
+                );
+
+                match key_type.as_ref() {
+                    // new key types can be added here
+                    DataType::UInt64 => generate_dict!(
+                        self,
+                        num_rows,
+                        num_distinct,
+                        null_pct,
+                        batch_gen_rng,
+                        array_gen_rng,
+                        UInt64Type,
+                        values
+                    ),
+                    _ => panic!("Invalid dictionary keys type: {key_type}"),
+                }
+            }
             _ => {
                 panic!("Unsupported data generator type: {}", col.column_type)
             }
@@ -636,8 +786,8 @@ mod tests {
         let batch1 = gen1.generate().unwrap();
         let batch2 = gen2.generate().unwrap();
 
-        let batch1_formatted = format!("{:?}", batch1);
-        let batch2_formatted = format!("{:?}", batch2);
+        let batch1_formatted = format!("{batch1:?}");
+        let batch2_formatted = format!("{batch2:?}");
 
         assert_eq!(batch1_formatted, batch2_formatted);
     }
