@@ -23,24 +23,28 @@ use std::time::SystemTime;
 use arrow::array::{ArrayRef, Int64Array, Int8Array, StringArray};
 use arrow::datatypes::{Field, Schema, SchemaBuilder};
 use arrow::record_batch::RecordBatch;
-use datafusion::assert_batches_sorted_eq;
 use datafusion::datasource::file_format::parquet::fetch_parquet_metadata;
 use datafusion::datasource::listing::PartitionedFile;
 use datafusion::datasource::object_store::ObjectStoreUrl;
 use datafusion::datasource::physical_plan::{
-    FileMeta, FileScanConfig, ParquetFileMetrics, ParquetFileReaderFactory, ParquetSource,
+    FileMeta, ParquetFileMetrics, ParquetFileReaderFactory, ParquetSource,
 };
 use datafusion::physical_plan::collect;
 use datafusion::physical_plan::metrics::ExecutionPlanMetricsSet;
 use datafusion::prelude::SessionContext;
+use datafusion_common::test_util::batches_to_sort_string;
 use datafusion_common::Result;
 
 use bytes::Bytes;
+use datafusion_datasource::file_scan_config::FileScanConfigBuilder;
+use datafusion_datasource::source::DataSourceExec;
 use futures::future::BoxFuture;
 use futures::{FutureExt, TryFutureExt};
+use insta::assert_snapshot;
 use object_store::memory::InMemory;
 use object_store::path::Path;
 use object_store::{ObjectMeta, ObjectStore};
+use parquet::arrow::arrow_reader::ArrowReaderOptions;
 use parquet::arrow::async_reader::AsyncFileReader;
 use parquet::arrow::ArrowWriter;
 use parquet::errors::ParquetError;
@@ -82,31 +86,30 @@ async fn route_data_access_ops_to_parquet_file_reader_factory() {
                 InMemoryParquetFileReaderFactory(Arc::clone(&in_memory_object_store)),
             )),
     );
-    let base_config = FileScanConfig::new(
+    let base_config = FileScanConfigBuilder::new(
         // just any url that doesn't point to in memory object store
         ObjectStoreUrl::local_filesystem(),
         file_schema,
         source,
     )
-    .with_file_group(file_group);
+    .with_file_group(file_group)
+    .build();
 
-    let parquet_exec = base_config.build();
+    let parquet_exec = DataSourceExec::from_data_source(base_config);
 
     let session_ctx = SessionContext::new();
     let task_ctx = session_ctx.task_ctx();
     let read = collect(parquet_exec, task_ctx).await.unwrap();
 
-    let expected = [
-        "+-----+----+----+",
-        "| c1  | c2 | c3 |",
-        "+-----+----+----+",
-        "| Foo | 1  | 10 |",
-        "|     | 2  | 20 |",
-        "| bar |    |    |",
-        "+-----+----+----+",
-    ];
-
-    assert_batches_sorted_eq!(expected, &read);
+    assert_snapshot!(batches_to_sort_string(&read), @r"
+    +-----+----+----+
+    | c1  | c2 | c3 |
+    +-----+----+----+
+    |     | 2  | 20 |
+    | Foo | 1  | 10 |
+    | bar |    |    |
+    +-----+----+----+
+    ");
 }
 
 #[derive(Debug)]
@@ -184,7 +187,7 @@ async fn store_parquet_in_memory(
                 location: Path::parse(format!("file-{offset}.parquet"))
                     .expect("creating path"),
                 last_modified: chrono::DateTime::from(SystemTime::now()),
-                size: buf.len(),
+                size: buf.len() as u64,
                 e_tag: None,
                 version: None,
             };
@@ -216,9 +219,10 @@ struct ParquetFileReader {
 impl AsyncFileReader for ParquetFileReader {
     fn get_bytes(
         &mut self,
-        range: Range<usize>,
+        range: Range<u64>,
     ) -> BoxFuture<'_, parquet::errors::Result<Bytes>> {
-        self.metrics.bytes_scanned.add(range.end - range.start);
+        let bytes_scanned = range.end - range.start;
+        self.metrics.bytes_scanned.add(bytes_scanned as usize);
 
         self.store
             .get_range(&self.meta.location, range)
@@ -230,6 +234,7 @@ impl AsyncFileReader for ParquetFileReader {
 
     fn get_metadata(
         &mut self,
+        _options: Option<&ArrowReaderOptions>,
     ) -> BoxFuture<'_, parquet::errors::Result<Arc<ParquetMetaData>>> {
         Box::pin(async move {
             let metadata = fetch_parquet_metadata(
