@@ -15,98 +15,31 @@
 // specific language governing permissions and limitations
 // under the License.
 
-//! Tests for the limited distinct aggregation optimizer rule
-
-use super::test_util::{parquet_exec_with_sort, schema, trim_plan_display};
+//! Integration tests for [`LimitedDistinctAggregation`] physical optimizer rule
 
 use std::sync::Arc;
 
-use arrow::{
-    array::Int32Array,
-    compute::SortOptions,
-    datatypes::{DataType, Field, Schema},
-    record_batch::RecordBatch,
-    util::pretty::pretty_format_batches,
+use crate::physical_optimizer::test_utils::{
+    assert_plan_matches_expected, build_group_by, mock_data, parquet_exec_with_sort,
+    schema, TestAggregate,
 };
-use arrow_schema::SchemaRef;
-use datafusion::{prelude::SessionContext, test_util::TestAggregate};
+
+use arrow::datatypes::DataType;
+use arrow::{compute::SortOptions, util::pretty::pretty_format_batches};
+use datafusion::prelude::SessionContext;
 use datafusion_common::Result;
 use datafusion_execution::config::SessionConfig;
 use datafusion_expr::Operator;
-use datafusion_physical_expr::{
-    expressions::{cast, col},
-    PhysicalExpr, PhysicalSortExpr,
-};
+use datafusion_physical_expr::expressions::cast;
+use datafusion_physical_expr::{expressions, expressions::col, PhysicalSortExpr};
 use datafusion_physical_expr_common::sort_expr::LexOrdering;
-use datafusion_physical_optimizer::{
-    limited_distinct_aggregation::LimitedDistinctAggregation, PhysicalOptimizerRule,
-};
 use datafusion_physical_plan::{
-    aggregates::{AggregateExec, AggregateMode, PhysicalGroupBy},
-    collect, displayable, expressions,
+    aggregates::{AggregateExec, AggregateMode},
+    collect,
     limit::{GlobalLimitExec, LocalLimitExec},
-    memory::MemoryExec,
     ExecutionPlan,
 };
 
-fn mock_data() -> Result<Arc<MemoryExec>> {
-    let schema = Arc::new(Schema::new(vec![
-        Field::new("a", DataType::Int32, true),
-        Field::new("b", DataType::Int32, true),
-    ]));
-
-    let batch = RecordBatch::try_new(
-        Arc::clone(&schema),
-        vec![
-            Arc::new(Int32Array::from(vec![
-                Some(1),
-                Some(2),
-                None,
-                Some(1),
-                Some(4),
-                Some(5),
-            ])),
-            Arc::new(Int32Array::from(vec![
-                Some(1),
-                None,
-                Some(6),
-                Some(2),
-                Some(8),
-                Some(9),
-            ])),
-        ],
-    )?;
-
-    Ok(Arc::new(MemoryExec::try_new(
-        &[vec![batch]],
-        Arc::clone(&schema),
-        None,
-    )?))
-}
-
-fn assert_plan_matches_expected(
-    plan: &Arc<dyn ExecutionPlan>,
-    expected: &[&str],
-) -> Result<()> {
-    let expected_lines: Vec<&str> = expected.to_vec();
-    let session_ctx = SessionContext::new();
-    let state = session_ctx.state();
-
-    let optimized = LimitedDistinctAggregation::new()
-        .optimize(Arc::clone(plan), state.config_options())?;
-
-    let optimized_result = displayable(optimized.as_ref()).indent(true).to_string();
-    let actual_lines = trim_plan_display(&optimized_result);
-
-    assert_eq!(
-        &expected_lines, &actual_lines,
-        "\n\nexpected:\n\n{:#?}\nactual:\n\n{:#?}\n\n",
-        expected_lines, actual_lines
-    );
-
-    Ok(())
-}
-
 async fn assert_results_match_expected(
     plan: Arc<dyn ExecutionPlan>,
     expected: &str,
@@ -119,20 +52,12 @@ async fn assert_results_match_expected(
     Ok(())
 }
 
-pub fn build_group_by(input_schema: &SchemaRef, columns: Vec<String>) -> PhysicalGroupBy {
-    let mut group_by_expr: Vec<(Arc<dyn PhysicalExpr>, String)> = vec![];
-    for column in columns.iter() {
-        group_by_expr.push((col(column, input_schema).unwrap(), column.to_string()));
-    }
-    PhysicalGroupBy::new_single(group_by_expr.clone())
-}
-
 #[tokio::test]
 async fn test_partial_final() -> Result<()> {
     let source = mock_data()?;
     let schema = source.schema();
 
-    // `SELECT a FROM MemoryExec GROUP BY a LIMIT 4;`, Partial/Final AggregateExec
+    // `SELECT a FROM DataSourceExec GROUP BY a LIMIT 4;`, Partial/Final AggregateExec
     let partial_agg = AggregateExec::try_new(
         AggregateMode::Partial,
         build_group_by(&schema.clone(), vec!["a".to_string()]),
@@ -158,7 +83,7 @@ async fn test_partial_final() -> Result<()> {
         "LocalLimitExec: fetch=4",
         "AggregateExec: mode=Final, gby=[a@0 as a], aggr=[], lim=[4]",
         "AggregateExec: mode=Partial, gby=[a@0 as a], aggr=[], lim=[4]",
-        "MemoryExec: partitions=1, partition_sizes=[1]",
+        "DataSourceExec: partitions=1, partition_sizes=[1]",
     ];
     let plan: Arc<dyn ExecutionPlan> = Arc::new(limit_exec);
     assert_plan_matches_expected(&plan, &expected)?;
@@ -182,7 +107,7 @@ async fn test_single_local() -> Result<()> {
     let source = mock_data()?;
     let schema = source.schema();
 
-    // `SELECT a FROM MemoryExec GROUP BY a LIMIT 4;`, Single AggregateExec
+    // `SELECT a FROM DataSourceExec GROUP BY a LIMIT 4;`, Single AggregateExec
     let single_agg = AggregateExec::try_new(
         AggregateMode::Single,
         build_group_by(&schema.clone(), vec!["a".to_string()]),
@@ -199,7 +124,7 @@ async fn test_single_local() -> Result<()> {
     let expected = [
         "LocalLimitExec: fetch=4",
         "AggregateExec: mode=Single, gby=[a@0 as a], aggr=[], lim=[4]",
-        "MemoryExec: partitions=1, partition_sizes=[1]",
+        "DataSourceExec: partitions=1, partition_sizes=[1]",
     ];
     let plan: Arc<dyn ExecutionPlan> = Arc::new(limit_exec);
     assert_plan_matches_expected(&plan, &expected)?;
@@ -223,7 +148,7 @@ async fn test_single_global() -> Result<()> {
     let source = mock_data()?;
     let schema = source.schema();
 
-    // `SELECT a FROM MemoryExec GROUP BY a LIMIT 4;`, Single AggregateExec
+    // `SELECT a FROM DataSourceExec GROUP BY a LIMIT 4;`, Single AggregateExec
     let single_agg = AggregateExec::try_new(
         AggregateMode::Single,
         build_group_by(&schema.clone(), vec!["a".to_string()]),
@@ -241,7 +166,7 @@ async fn test_single_global() -> Result<()> {
     let expected = [
         "GlobalLimitExec: skip=1, fetch=3",
         "AggregateExec: mode=Single, gby=[a@0 as a], aggr=[], lim=[4]",
-        "MemoryExec: partitions=1, partition_sizes=[1]",
+        "DataSourceExec: partitions=1, partition_sizes=[1]",
     ];
     let plan: Arc<dyn ExecutionPlan> = Arc::new(limit_exec);
     assert_plan_matches_expected(&plan, &expected)?;
@@ -264,7 +189,7 @@ async fn test_distinct_cols_different_than_group_by_cols() -> Result<()> {
     let source = mock_data()?;
     let schema = source.schema();
 
-    // `SELECT distinct a FROM MemoryExec GROUP BY a, b LIMIT 4;`, Single/Single AggregateExec
+    // `SELECT distinct a FROM DataSourceExec GROUP BY a, b LIMIT 4;`, Single/Single AggregateExec
     let group_by_agg = AggregateExec::try_new(
         AggregateMode::Single,
         build_group_by(&schema.clone(), vec!["a".to_string(), "b".to_string()]),
@@ -290,7 +215,7 @@ async fn test_distinct_cols_different_than_group_by_cols() -> Result<()> {
         "LocalLimitExec: fetch=4",
         "AggregateExec: mode=Single, gby=[a@0 as a], aggr=[], lim=[4]",
         "AggregateExec: mode=Single, gby=[a@0 as a, b@1 as b], aggr=[]",
-        "MemoryExec: partitions=1, partition_sizes=[1]",
+        "DataSourceExec: partitions=1, partition_sizes=[1]",
     ];
     let plan: Arc<dyn ExecutionPlan> = Arc::new(limit_exec);
     assert_plan_matches_expected(&plan, &expected)?;
@@ -309,12 +234,46 @@ async fn test_distinct_cols_different_than_group_by_cols() -> Result<()> {
     Ok(())
 }
 
+#[test]
+fn test_has_order_by() -> Result<()> {
+    let sort_key = LexOrdering::new(vec![PhysicalSortExpr {
+        expr: col("a", &schema()).unwrap(),
+        options: SortOptions::default(),
+    }]);
+    let source = parquet_exec_with_sort(vec![sort_key]);
+    let schema = source.schema();
+
+    // `SELECT a FROM DataSourceExec WHERE a > 1 GROUP BY a LIMIT 10;`, Single AggregateExec
+    // the `a > 1` filter is applied in the AggregateExec
+    let single_agg = AggregateExec::try_new(
+        AggregateMode::Single,
+        build_group_by(&schema, vec!["a".to_string()]),
+        vec![], /* aggr_expr */
+        vec![], /* filter_expr */
+        source, /* input */
+        schema, /* input_schema */
+    )?;
+    let limit_exec = LocalLimitExec::new(
+        Arc::new(single_agg),
+        10, // fetch
+    );
+    // expected not to push the limit to the AggregateExec
+    let expected = [
+        "LocalLimitExec: fetch=10",
+        "AggregateExec: mode=Single, gby=[a@0 as a], aggr=[], ordering_mode=Sorted",
+        "DataSourceExec: file_groups={1 group: [[x]]}, projection=[a, b, c, d, e], output_ordering=[a@0 ASC], file_type=parquet",
+    ];
+    let plan: Arc<dyn ExecutionPlan> = Arc::new(limit_exec);
+    assert_plan_matches_expected(&plan, &expected)?;
+    Ok(())
+}
+
 #[test]
 fn test_no_group_by() -> Result<()> {
     let source = mock_data()?;
     let schema = source.schema();
 
-    // `SELECT <aggregate with no expressions> FROM MemoryExec LIMIT 10;`, Single AggregateExec
+    // `SELECT <aggregate with no expressions> FROM DataSourceExec LIMIT 10;`, Single AggregateExec
     let single_agg = AggregateExec::try_new(
         AggregateMode::Single,
         build_group_by(&schema, vec![]),
@@ -331,7 +290,7 @@ fn test_no_group_by() -> Result<()> {
     let expected = [
         "LocalLimitExec: fetch=10",
         "AggregateExec: mode=Single, gby=[], aggr=[]",
-        "MemoryExec: partitions=1, partition_sizes=[1]",
+        "DataSourceExec: partitions=1, partition_sizes=[1]",
     ];
     let plan: Arc<dyn ExecutionPlan> = Arc::new(limit_exec);
     assert_plan_matches_expected(&plan, &expected)?;
@@ -344,7 +303,7 @@ fn test_has_aggregate_expression() -> Result<()> {
     let schema = source.schema();
     let agg = TestAggregate::new_count_star();
 
-    // `SELECT <aggregate with no expressions> FROM MemoryExec LIMIT 10;`, Single AggregateExec
+    // `SELECT <aggregate with no expressions> FROM DataSourceExec LIMIT 10;`, Single AggregateExec
     let single_agg = AggregateExec::try_new(
         AggregateMode::Single,
         build_group_by(&schema, vec!["a".to_string()]),
@@ -361,7 +320,7 @@ fn test_has_aggregate_expression() -> Result<()> {
     let expected = [
         "LocalLimitExec: fetch=10",
         "AggregateExec: mode=Single, gby=[a@0 as a], aggr=[COUNT(*)]",
-        "MemoryExec: partitions=1, partition_sizes=[1]",
+        "DataSourceExec: partitions=1, partition_sizes=[1]",
     ];
     let plan: Arc<dyn ExecutionPlan> = Arc::new(limit_exec);
     assert_plan_matches_expected(&plan, &expected)?;
@@ -373,7 +332,7 @@ fn test_has_filter() -> Result<()> {
     let source = mock_data()?;
     let schema = source.schema();
 
-    // `SELECT a FROM MemoryExec WHERE a > 1 GROUP BY a LIMIT 10;`, Single AggregateExec
+    // `SELECT a FROM DataSourceExec WHERE a > 1 GROUP BY a LIMIT 10;`, Single AggregateExec
     // the `a > 1` filter is applied in the AggregateExec
     let filter_expr = Some(expressions::binary(
         col("a", &schema)?,
@@ -399,43 +358,9 @@ fn test_has_filter() -> Result<()> {
     let expected = [
         "LocalLimitExec: fetch=10",
         "AggregateExec: mode=Single, gby=[a@0 as a], aggr=[COUNT(*)]",
-        "MemoryExec: partitions=1, partition_sizes=[1]",
+        "DataSourceExec: partitions=1, partition_sizes=[1]",
     ];
     let plan: Arc<dyn ExecutionPlan> = Arc::new(limit_exec);
     assert_plan_matches_expected(&plan, &expected)?;
     Ok(())
 }
-
-#[test]
-fn test_has_order_by() -> Result<()> {
-    let sort_key = LexOrdering::new(vec![PhysicalSortExpr {
-        expr: col("a", &schema()).unwrap(),
-        options: SortOptions::default(),
-    }]);
-    let source = parquet_exec_with_sort(vec![sort_key]);
-    let schema = source.schema();
-
-    // `SELECT a FROM MemoryExec WHERE a > 1 GROUP BY a LIMIT 10;`, Single AggregateExec
-    // the `a > 1` filter is applied in the AggregateExec
-    let single_agg = AggregateExec::try_new(
-        AggregateMode::Single,
-        build_group_by(&schema, vec!["a".to_string()]),
-        vec![], /* aggr_expr */
-        vec![], /* filter_expr */
-        source, /* input */
-        schema, /* input_schema */
-    )?;
-    let limit_exec = LocalLimitExec::new(
-        Arc::new(single_agg),
-        10, // fetch
-    );
-    // expected not to push the limit to the AggregateExec
-    let expected = [
-            "LocalLimitExec: fetch=10",
-            "AggregateExec: mode=Single, gby=[a@0 as a], aggr=[], ordering_mode=Sorted",
-            "ParquetExec: file_groups={1 group: [[x]]}, projection=[a, b, c, d, e], output_ordering=[a@0 ASC]",
-        ];
-    let plan: Arc<dyn ExecutionPlan> = Arc::new(limit_exec);
-    assert_plan_matches_expected(&plan, &expected)?;
-    Ok(())
-}
