@@ -78,7 +78,7 @@ impl FileStream {
         file_opener: Arc<dyn FileOpener>,
         metrics: &ExecutionPlanMetricsSet,
     ) -> Result<Self> {
-        let (projected_schema, ..) = config.project();
+        let projected_schema = config.projected_schema();
         let pc_projector = PartitionColumnProjector::new(
             Arc::clone(&projected_schema),
             &config
