@@ -137,7 +137,9 @@ impl MinMaxStatistics {
                 // Reverse the projection to get the index of the column in the full statistics
                 // The file statistics contains _every_ column , but the sort column's index()
                 // refers to the index in projected_schema
-                let i = projection.map(|p| p[c.index()]).unwrap_or(c.index());
+                let i = projection
+                    .map(|p| p[c.index()])
+                    .unwrap_or_else(|| c.index());
 
                 let (min, max) = get_min_max(i).map_err(|e| {
                     e.context(format!("get min/max for column: '{}'", c.name()))
@@ -476,7 +478,7 @@ pub fn compute_all_files_statistics(
     // Then summary statistics across all file groups
     let file_groups_statistics = file_groups_with_stats
         .iter()
-        .filter_map(|file_group| file_group.statistics());
+        .filter_map(|file_group| file_group.file_statistics(None));
 
     let mut statistics =
         Statistics::try_merge_iter(file_groups_statistics, &table_schema)?;
