@@ -17,25 +17,22 @@
 
 //! [`ScalarUDFImpl`] definitions for map_extract functions.
 
-use arrow::array::{ArrayRef, Capacities, MutableArrayData};
-use arrow_array::{make_array, ListArray};
-
-use arrow::datatypes::DataType;
-use arrow_array::{Array, MapArray};
-use arrow_buffer::OffsetBuffer;
-use arrow_schema::Field;
-
+use crate::utils::{get_map_entry_field, make_scalar_function};
+use arrow::array::{
+    make_array, Array, ArrayRef, Capacities, ListArray, MapArray, MutableArrayData,
+};
+use arrow::buffer::OffsetBuffer;
+use arrow::datatypes::{DataType, Field};
+use datafusion_common::utils::take_function_args;
 use datafusion_common::{cast::as_map_array, exec_err, Result};
-use datafusion_expr::scalar_doc_sections::DOC_SECTION_MAP;
 use datafusion_expr::{
     ColumnarValue, Documentation, ScalarUDFImpl, Signature, Volatility,
 };
+use datafusion_macros::user_doc;
 use std::any::Any;
-use std::sync::{Arc, OnceLock};
+use std::sync::Arc;
 use std::vec;
 
-use crate::utils::{get_map_entry_field, make_scalar_function};
-
 // Create static instances of ScalarUDFs for each function
 make_udf_expr_and_func!(
     MapExtract,
@@ -45,12 +42,44 @@ make_udf_expr_and_func!(
     map_extract_udf
 );
 
+#[user_doc(
+    doc_section(label = "Map Functions"),
+    description = "Returns a list containing the value for the given key or an empty list if the key is not present in the map.",
+    syntax_example = "map_extract(map, key)",
+    sql_example = r#"```sql
+SELECT map_extract(MAP {'a': 1, 'b': NULL, 'c': 3}, 'a');
+----
+[1]
+
+SELECT map_extract(MAP {1: 'one', 2: 'two'}, 2);
+----
+['two']
+
+SELECT map_extract(MAP {'x': 10, 'y': NULL, 'z': 30}, 'y');
+----
+[]
+```"#,
+    argument(
+        name = "map",
+        description = "Map expression. Can be a constant, column, or function, and any combination of map operators."
+    ),
+    argument(
+        name = "key",
+        description = "Key to extract from the map. Can be a constant, column, or function, any combination of arithmetic or string operators, or a named expression of the previously listed."
+    )
+)]
 #[derive(Debug)]
-pub(super) struct MapExtract {
+pub struct MapExtract {
     signature: Signature,
     aliases: Vec<String>,
 }
 
+impl Default for MapExtract {
+    fn default() -> Self {
+        Self::new()
+    }
+}
+
 impl MapExtract {
     pub fn new() -> Self {
         Self {
@@ -73,10 +102,7 @@ impl ScalarUDFImpl for MapExtract {
     }
 
     fn return_type(&self, arg_types: &[DataType]) -> Result<DataType> {
-        if arg_types.len() != 2 {
-            return exec_err!("map_extract expects two arguments");
-        }
-        let map_type = &arg_types[0];
+        let [map_type, _] = take_function_args(self.name(), arg_types)?;
         let map_fields = get_map_entry_field(map_type)?;
         Ok(DataType::List(Arc::new(Field::new_list_field(
             map_fields.last().unwrap().data_type().clone(),
@@ -84,12 +110,11 @@ impl ScalarUDFImpl for MapExtract {
         ))))
     }
 
-    fn invoke_batch(
+    fn invoke_with_args(
         &self,
-        args: &[ColumnarValue],
-        _number_rows: usize,
+        args: datafusion_expr::ScalarFunctionArgs,
     ) -> Result<ColumnarValue> {
-        make_scalar_function(map_extract_inner)(args)
+        make_scalar_function(map_extract_inner)(&args.args)
     }
 
     fn aliases(&self) -> &[String] {
@@ -97,57 +122,20 @@ impl ScalarUDFImpl for MapExtract {
     }
 
     fn coerce_types(&self, arg_types: &[DataType]) -> Result<Vec<DataType>> {
-        if arg_types.len() != 2 {
-            return exec_err!("map_extract expects two arguments");
-        }
+        let [map_type, _] = take_function_args(self.name(), arg_types)?;
 
-        let field = get_map_entry_field(&arg_types[0])?;
+        let field = get_map_entry_field(map_type)?;
         Ok(vec![
-            arg_types[0].clone(),
+            map_type.clone(),
             field.first().unwrap().data_type().clone(),
         ])
     }
 
     fn documentation(&self) -> Option<&Documentation> {
-        Some(get_map_extract_doc())
+        self.doc()
     }
 }
 
-static DOCUMENTATION: OnceLock<Documentation> = OnceLock::new();
-
-fn get_map_extract_doc() -> &'static Documentation {
-    DOCUMENTATION.get_or_init(|| {
-        Documentation::builder(
-            DOC_SECTION_MAP,
-                "Returns a list containing the value for the given key or an empty list if the key is not present in the map.",
-            "map_extract(map, key)")
-            .with_sql_example(
-                r#"```sql
-SELECT map_extract(MAP {'a': 1, 'b': NULL, 'c': 3}, 'a');
-----
-[1]
-
-SELECT map_extract(MAP {1: 'one', 2: 'two'}, 2);
-----
-['two']
-
-SELECT map_extract(MAP {'x': 10, 'y': NULL, 'z': 30}, 'y');
-----
-[]
-```"#,
-            )
-            .with_argument(
-                "map",
-                "Map expression. Can be a constant, column, or function, and any combination of map operators.",
-            )
-            .with_argument(
-                "key",
-                "Key to extract from the map. Can be a constant, column, or function, any combination of arithmetic or string operators, or a named expression of the previously listed.",
-            )
-            .build()
-    })
-}
-
 fn general_map_extract_inner(
     map_array: &MapArray,
     query_keys_array: &dyn Array,
@@ -194,24 +182,22 @@ fn general_map_extract_inner(
 }
 
 fn map_extract_inner(args: &[ArrayRef]) -> Result<ArrayRef> {
-    if args.len() != 2 {
-        return exec_err!("map_extract expects two arguments");
-    }
+    let [map_arg, key_arg] = take_function_args("map_extract", args)?;
 
-    let map_array = match args[0].data_type() {
-        DataType::Map(_, _) => as_map_array(&args[0])?,
+    let map_array = match map_arg.data_type() {
+        DataType::Map(_, _) => as_map_array(&map_arg)?,
         _ => return exec_err!("The first argument in map_extract must be a map"),
     };
 
     let key_type = map_array.key_type();
 
-    if key_type != args[1].data_type() {
+    if key_type != key_arg.data_type() {
         return exec_err!(
             "The key type {} does not match the map key type {}",
-            args[1].data_type(),
+            key_arg.data_type(),
             key_type
         );
     }
 
-    general_map_extract_inner(map_array, &args[1])
+    general_map_extract_inner(map_array, key_arg)
 }
