@@ -21,11 +21,11 @@ use crate::utils::make_scalar_function;
 use arrow::array::{new_null_array, Array, ArrayRef, ListArray, NullBufferBuilder};
 use arrow::buffer::OffsetBuffer;
 use arrow::compute::SortColumn;
-use arrow::datatypes::DataType::{FixedSizeList, LargeList, List};
 use arrow::datatypes::{DataType, Field};
 use arrow::{compute, compute::SortOptions};
 use datafusion_common::cast::{as_list_array, as_string_array};
-use datafusion_common::{exec_err, Result};
+use datafusion_common::utils::ListCoercion;
+use datafusion_common::{exec_err, plan_err, Result};
 use datafusion_expr::{
     ArrayFunctionArgument, ArrayFunctionSignature, ColumnarValue, Documentation,
     ScalarUDFImpl, Signature, TypeSignature, Volatility,
@@ -93,14 +93,14 @@ impl ArraySort {
                 vec![
                     TypeSignature::ArraySignature(ArrayFunctionSignature::Array {
                         arguments: vec![ArrayFunctionArgument::Array],
-                        array_coercion: None,
+                        array_coercion: Some(ListCoercion::FixedSizedListToList),
                     }),
                     TypeSignature::ArraySignature(ArrayFunctionSignature::Array {
                         arguments: vec![
                             ArrayFunctionArgument::Array,
                             ArrayFunctionArgument::String,
                         ],
-                        array_coercion: None,
+                        array_coercion: Some(ListCoercion::FixedSizedListToList),
                     }),
                     TypeSignature::ArraySignature(ArrayFunctionSignature::Array {
                         arguments: vec![
@@ -108,7 +108,7 @@ impl ArraySort {
                             ArrayFunctionArgument::String,
                             ArrayFunctionArgument::String,
                         ],
-                        array_coercion: None,
+                        array_coercion: Some(ListCoercion::FixedSizedListToList),
                     }),
                 ],
                 Volatility::Immutable,
@@ -133,17 +133,13 @@ impl ScalarUDFImpl for ArraySort {
 
     fn return_type(&self, arg_types: &[DataType]) -> Result<DataType> {
         match &arg_types[0] {
-            List(field) | FixedSizeList(field, _) => Ok(List(Arc::new(
-                Field::new_list_field(field.data_type().clone(), true),
-            ))),
-            LargeList(field) => Ok(LargeList(Arc::new(Field::new_list_field(
-                field.data_type().clone(),
-                true,
-            )))),
             DataType::Null => Ok(DataType::Null),
-            _ => exec_err!(
-                "Not reachable, data_type should be List, LargeList or FixedSizeList"
-            ),
+            DataType::List(field) => {
+                Ok(DataType::new_list(field.data_type().clone(), true))
+            }
+            arg_type => {
+                plan_err!("{} does not support type {arg_type}", self.name())
+            }
         }
     }
 
@@ -169,6 +165,16 @@ pub fn array_sort_inner(args: &[ArrayRef]) -> Result<ArrayRef> {
         return exec_err!("array_sort expects one to three arguments");
     }
 
+    if args[0].data_type().is_null() {
+        return Ok(Arc::clone(&args[0]));
+    }
+
+    let list_array = as_list_array(&args[0])?;
+    let row_count = list_array.len();
+    if row_count == 0 || list_array.value_type().is_null() {
+        return Ok(Arc::clone(&args[0]));
+    }
+
     if args[1..].iter().any(|array| array.is_null(0)) {
         return Ok(new_null_array(args[0].data_type(), args[0].len()));
     }
@@ -193,12 +199,6 @@ pub fn array_sort_inner(args: &[ArrayRef]) -> Result<ArrayRef> {
         _ => return exec_err!("array_sort expects 1 to 3 arguments"),
     };
 
-    let list_array = as_list_array(&args[0])?;
-    let row_count = list_array.len();
-    if row_count == 0 {
-        return Ok(Arc::clone(&args[0]));
-    }
-
     let mut array_lengths = vec![];
     let mut arrays = vec![];
     let mut valid = NullBufferBuilder::new(row_count);
