@@ -23,7 +23,7 @@ use datafusion_expr::{ColumnarValue, ScalarFunctionArgs};
 use datafusion_functions::string::chr;
 use rand::{Rng, SeedableRng};
 
-use arrow::datatypes::DataType;
+use arrow::datatypes::{DataType, Field};
 use rand::rngs::StdRng;
 use std::sync::Arc;
 
@@ -37,27 +37,34 @@ fn criterion_benchmark(c: &mut Criterion) {
     let size = 1024;
     let input: PrimitiveArray<Int64Type> = {
         let null_density = 0.2;
-        let mut rng = seedable_rng();
+        let mut rng = StdRng::seed_from_u64(42);
         (0..size)
             .map(|_| {
-                if rng.gen::<f32>() < null_density {
+                if rng.random::<f32>() < null_density {
                     None
                 } else {
-                    Some(rng.gen_range::<i64, _>(1i64..10_000))
+                    Some(rng.random_range::<i64, _>(1i64..10_000))
                 }
             })
             .collect()
     };
     let input = Arc::new(input);
     let args = vec![ColumnarValue::Array(input)];
+    let arg_fields = args
+        .iter()
+        .enumerate()
+        .map(|(idx, arg)| Field::new(format!("arg_{idx}"), arg.data_type(), true).into())
+        .collect::<Vec<_>>();
+
     c.bench_function("chr", |b| {
         b.iter(|| {
             black_box(
                 cot_fn
                     .invoke_with_args(ScalarFunctionArgs {
                         args: args.clone(),
+                        arg_fields: arg_fields.clone(),
                         number_rows: size,
-                        return_type: &DataType::Utf8,
+                        return_field: Field::new("f", DataType::Utf8, true).into(),
                     })
                     .unwrap(),
             )
