@@ -18,11 +18,12 @@
 extern crate criterion;
 
 use arrow::array::{ArrayRef, Int64Array, OffsetSizeTrait};
-use arrow::datatypes::DataType;
+use arrow::datatypes::{DataType, Field};
 use arrow::util::bench_util::{
     create_string_array_with_len, create_string_view_array_with_len,
 };
 use criterion::{black_box, criterion_group, criterion_main, Criterion, SamplingMode};
+use datafusion_common::DataFusionError;
 use datafusion_expr::{ColumnarValue, ScalarFunctionArgs};
 use datafusion_functions::unicode;
 use std::sync::Arc;
@@ -96,8 +97,25 @@ fn create_args_with_count<O: OffsetSizeTrait>(
     }
 }
 
+fn invoke_substr_with_args(
+    args: Vec<ColumnarValue>,
+    number_rows: usize,
+) -> Result<ColumnarValue, DataFusionError> {
+    let arg_fields = args
+        .iter()
+        .enumerate()
+        .map(|(idx, arg)| Field::new(format!("arg_{idx}"), arg.data_type(), true).into())
+        .collect::<Vec<_>>();
+
+    unicode::substr().invoke_with_args(ScalarFunctionArgs {
+        args: args.clone(),
+        arg_fields,
+        number_rows,
+        return_field: Field::new("f", DataType::Utf8View, true).into(),
+    })
+}
+
 fn criterion_benchmark(c: &mut Criterion) {
-    let substr = unicode::substr();
     for size in [1024, 4096] {
         // string_len = 12, substring_len=6 (see `create_args_without_count`)
         let len = 12;
@@ -107,44 +125,19 @@ fn criterion_benchmark(c: &mut Criterion) {
 
         let args = create_args_without_count::<i32>(size, len, true, true);
         group.bench_function(
-            format!("substr_string_view [size={}, strlen={}]", size, len),
-            |b| {
-                b.iter(|| {
-                    black_box(substr.invoke_with_args(ScalarFunctionArgs {
-                        args: args.clone(),
-                        number_rows: size,
-                        return_type: &DataType::Utf8View,
-                    }))
-                })
-            },
+            format!("substr_string_view [size={size}, strlen={len}]"),
+            |b| b.iter(|| black_box(invoke_substr_with_args(args.clone(), size))),
         );
 
         let args = create_args_without_count::<i32>(size, len, false, false);
-        group.bench_function(
-            format!("substr_string [size={}, strlen={}]", size, len),
-            |b| {
-                b.iter(|| {
-                    black_box(substr.invoke_with_args(ScalarFunctionArgs {
-                        args: args.clone(),
-                        number_rows: size,
-                        return_type: &DataType::Utf8View,
-                    }))
-                })
-            },
-        );
+        group.bench_function(format!("substr_string [size={size}, strlen={len}]"), |b| {
+            b.iter(|| black_box(invoke_substr_with_args(args.clone(), size)))
+        });
 
         let args = create_args_without_count::<i64>(size, len, true, false);
         group.bench_function(
-            format!("substr_large_string [size={}, strlen={}]", size, len),
-            |b| {
-                b.iter(|| {
-                    black_box(substr.invoke_with_args(ScalarFunctionArgs {
-                        args: args.clone(),
-                        number_rows: size,
-                        return_type: &DataType::Utf8View,
-                    }))
-                })
-            },
+            format!("substr_large_string [size={size}, strlen={len}]"),
+            |b| b.iter(|| black_box(invoke_substr_with_args(args.clone(), size))),
         );
 
         group.finish();
@@ -158,53 +151,20 @@ fn criterion_benchmark(c: &mut Criterion) {
 
         let args = create_args_with_count::<i32>(size, len, count, true);
         group.bench_function(
-            format!(
-                "substr_string_view [size={}, count={}, strlen={}]",
-                size, count, len,
-            ),
-            |b| {
-                b.iter(|| {
-                    black_box(substr.invoke_with_args(ScalarFunctionArgs {
-                        args: args.clone(),
-                        number_rows: size,
-                        return_type: &DataType::Utf8View,
-                    }))
-                })
-            },
+            format!("substr_string_view [size={size}, count={count}, strlen={len}]",),
+            |b| b.iter(|| black_box(invoke_substr_with_args(args.clone(), size))),
         );
 
         let args = create_args_with_count::<i32>(size, len, count, false);
         group.bench_function(
-            format!(
-                "substr_string [size={}, count={}, strlen={}]",
-                size, count, len,
-            ),
-            |b| {
-                b.iter(|| {
-                    black_box(substr.invoke_with_args(ScalarFunctionArgs {
-                        args: args.clone(),
-                        number_rows: size,
-                        return_type: &DataType::Utf8View,
-                    }))
-                })
-            },
+            format!("substr_string [size={size}, count={count}, strlen={len}]",),
+            |b| b.iter(|| black_box(invoke_substr_with_args(args.clone(), size))),
         );
 
         let args = create_args_with_count::<i64>(size, len, count, false);
         group.bench_function(
-            format!(
-                "substr_large_string [size={}, count={}, strlen={}]",
-                size, count, len,
-            ),
-            |b| {
-                b.iter(|| {
-                    black_box(substr.invoke_with_args(ScalarFunctionArgs {
-                        args: args.clone(),
-                        number_rows: size,
-                        return_type: &DataType::Utf8View,
-                    }))
-                })
-            },
+            format!("substr_large_string [size={size}, count={count}, strlen={len}]",),
+            |b| b.iter(|| black_box(invoke_substr_with_args(args.clone(), size))),
         );
 
         group.finish();
@@ -218,53 +178,20 @@ fn criterion_benchmark(c: &mut Criterion) {
 
         let args = create_args_with_count::<i32>(size, len, count, true);
         group.bench_function(
-            format!(
-                "substr_string_view [size={}, count={}, strlen={}]",
-                size, count, len,
-            ),
-            |b| {
-                b.iter(|| {
-                    black_box(substr.invoke_with_args(ScalarFunctionArgs {
-                        args: args.clone(),
-                        number_rows: size,
-                        return_type: &DataType::Utf8View,
-                    }))
-                })
-            },
+            format!("substr_string_view [size={size}, count={count}, strlen={len}]",),
+            |b| b.iter(|| black_box(invoke_substr_with_args(args.clone(), size))),
         );
 
         let args = create_args_with_count::<i32>(size, len, count, false);
         group.bench_function(
-            format!(
-                "substr_string [size={}, count={}, strlen={}]",
-                size, count, len,
-            ),
-            |b| {
-                b.iter(|| {
-                    black_box(substr.invoke_with_args(ScalarFunctionArgs {
-                        args: args.clone(),
-                        number_rows: size,
-                        return_type: &DataType::Utf8View,
-                    }))
-                })
-            },
+            format!("substr_string [size={size}, count={count}, strlen={len}]",),
+            |b| b.iter(|| black_box(invoke_substr_with_args(args.clone(), size))),
         );
 
         let args = create_args_with_count::<i64>(size, len, count, false);
         group.bench_function(
-            format!(
-                "substr_large_string [size={}, count={}, strlen={}]",
-                size, count, len,
-            ),
-            |b| {
-                b.iter(|| {
-                    black_box(substr.invoke_with_args(ScalarFunctionArgs {
-                        args: args.clone(),
-                        number_rows: size,
-                        return_type: &DataType::Utf8View,
-                    }))
-                })
-            },
+            format!("substr_large_string [size={size}, count={count}, strlen={len}]",),
+            |b| b.iter(|| black_box(invoke_substr_with_args(args.clone(), size))),
         );
 
         group.finish();
