@@ -37,13 +37,14 @@ use itertools::Itertools;
 /// Example:
 /// ```
 /// # use std::any::Any;
+/// # use std::collections::HashMap;
 /// # use std::fmt::{Display, Formatter};
 /// # use std::hash::Hasher;
 /// # use std::sync::Arc;
 /// # use arrow::array::RecordBatch;
 /// # use datafusion_common::Result;
 /// # use arrow::compute::SortOptions;
-/// # use arrow::datatypes::{DataType, Schema};
+/// # use arrow::datatypes::{DataType, Field, FieldRef, Schema};
 /// # use datafusion_expr_common::columnar_value::ColumnarValue;
 /// # use datafusion_physical_expr_common::physical_expr::PhysicalExpr;
 /// # use datafusion_physical_expr_common::sort_expr::PhysicalSortExpr;
@@ -56,6 +57,7 @@ use itertools::Itertools;
 /// #  fn data_type(&self, input_schema: &Schema) -> Result<DataType> {todo!()}
 /// #  fn nullable(&self, input_schema: &Schema) -> Result<bool> {todo!() }
 /// #  fn evaluate(&self, batch: &RecordBatch) -> Result<ColumnarValue> {todo!() }
+/// #  fn return_field(&self, input_schema: &Schema) -> Result<FieldRef> { unimplemented!() }
 /// #  fn children(&self) -> Vec<&Arc<dyn PhysicalExpr>> {todo!()}
 /// #  fn with_new_children(self: Arc<Self>, children: Vec<Arc<dyn PhysicalExpr>>) -> Result<Arc<dyn PhysicalExpr>> {todo!()}
 /// # fn fmt_sql(&self, f: &mut Formatter<'_>) -> std::fmt::Result { todo!() }
@@ -265,10 +267,10 @@ pub fn format_physical_sort_requirement_list(
             let mut iter = self.0.iter();
             write!(f, "[")?;
             if let Some(expr) = iter.next() {
-                write!(f, "{}", expr)?;
+                write!(f, "{expr}")?;
             }
             for expr in iter {
-                write!(f, ", {}", expr)?;
+                write!(f, ", {expr}")?;
             }
             write!(f, "]")?;
             Ok(())
@@ -508,7 +510,7 @@ impl Display for LexOrdering {
             } else {
                 write!(f, ", ")?;
             }
-            write!(f, "{}", sort_expr)?;
+            write!(f, "{sort_expr}")?;
         }
         Ok(())
     }
