@@ -195,7 +195,7 @@ struct AscendingRandomFloatIterator {
 impl AscendingRandomFloatIterator {
     fn new(min: f64, max: f64) -> Self {
         let mut rng = StdRng::seed_from_u64(42);
-        let initial = rng.gen_range(min..max);
+        let initial = rng.random_range(min..max);
         AscendingRandomFloatIterator {
             prev: initial,
             max,
@@ -208,7 +208,7 @@ impl Iterator for AscendingRandomFloatIterator {
     type Item = f64;
 
     fn next(&mut self) -> Option<Self::Item> {
-        let value = self.rng.gen_range(self.prev..self.max);
+        let value = self.rng.random_range(self.prev..self.max);
         self.prev = value;
         Some(value)
     }
