@@ -110,7 +110,7 @@ impl DisplayAs for GlobalLimitExec {
             }
             DisplayFormatType::TreeRender => {
                 if let Some(fetch) = self.fetch {
-                    writeln!(f, "limit={}", fetch)?;
+                    writeln!(f, "limit={fetch}")?;
                 }
                 write!(f, "skip={}", self.skip)
             }
@@ -164,10 +164,7 @@ impl ExecutionPlan for GlobalLimitExec {
         partition: usize,
         context: Arc<TaskContext>,
     ) -> Result<SendableRecordBatchStream> {
-        trace!(
-            "Start GlobalLimitExec::execute for partition: {}",
-            partition
-        );
+        trace!("Start GlobalLimitExec::execute for partition: {partition}");
         // GlobalLimitExec has a single output partition
         if 0 != partition {
             return internal_err!("GlobalLimitExec invalid partition {partition}");
@@ -193,8 +190,11 @@ impl ExecutionPlan for GlobalLimitExec {
     }
 
     fn statistics(&self) -> Result<Statistics> {
-        Statistics::with_fetch(
-            self.input.statistics()?,
+        self.partition_statistics(None)
+    }
+
+    fn partition_statistics(&self, partition: Option<usize>) -> Result<Statistics> {
+        self.input.partition_statistics(partition)?.with_fetch(
             self.schema(),
             self.fetch,
             self.skip,
@@ -334,8 +334,11 @@ impl ExecutionPlan for LocalLimitExec {
     }
 
     fn statistics(&self) -> Result<Statistics> {
-        Statistics::with_fetch(
-            self.input.statistics()?,
+        self.partition_statistics(None)
+    }
+
+    fn partition_statistics(&self, partition: Option<usize>) -> Result<Statistics> {
+        self.input.partition_statistics(partition)?.with_fetch(
             self.schema(),
             Some(self.fetch),
             0,
@@ -765,7 +768,7 @@ mod tests {
         let offset =
             GlobalLimitExec::new(Arc::new(CoalescePartitionsExec::new(csv)), skip, fetch);
 
-        Ok(offset.statistics()?.num_rows)
+        Ok(offset.partition_statistics(None)?.num_rows)
     }
 
     pub fn build_group_by(
@@ -805,7 +808,7 @@ mod tests {
             fetch,
         );
 
-        Ok(offset.statistics()?.num_rows)
+        Ok(offset.partition_statistics(None)?.num_rows)
     }
 
     async fn row_number_statistics_for_local_limit(
@@ -818,7 +821,7 @@ mod tests {
 
         let offset = LocalLimitExec::new(csv, fetch);
 
-        Ok(offset.statistics()?.num_rows)
+        Ok(offset.partition_statistics(None)?.num_rows)
     }
 
     /// Return a RecordBatch with a single array with row_count sz
