@@ -15,7 +15,7 @@
 // specific language governing permissions and limitations
 // under the License.
 
-use arrow::datatypes::{DataType, Field};
+use arrow::datatypes::{DataType, Field, FieldRef};
 use datafusion::logical_expr::ColumnarValue;
 use datafusion_common::plan_err;
 use datafusion_expr::function::AccumulatorArgs;
@@ -166,8 +166,11 @@ impl WindowUDFImpl for CustomUDWF {
         Ok(Box::new(CustomUDWFEvaluator {}))
     }
 
-    fn field(&self, field_args: WindowUDFFieldArgs) -> datafusion_common::Result<Field> {
-        Ok(Field::new(field_args.name(), DataType::UInt64, false))
+    fn field(
+        &self,
+        field_args: WindowUDFFieldArgs,
+    ) -> datafusion_common::Result<FieldRef> {
+        Ok(Field::new(field_args.name(), DataType::UInt64, false).into())
     }
 }
 
