@@ -83,7 +83,7 @@ fn udf_roundtrip_with_registry() {
 
 #[test]
 #[should_panic(
-    expected = "No function registry provided to deserialize, so can not deserialize User Defined Function 'dummy'"
+    expected = "LogicalExtensionCodec is not provided for scalar function dummy"
 )]
 fn udf_roundtrip_without_registry() {
     let ctx = context_with_udf();
@@ -256,7 +256,7 @@ fn test_expression_serialization_roundtrip() {
     use datafusion_proto::logical_plan::from_proto::parse_expr;
 
     let ctx = SessionContext::new();
-    let lit = Expr::Literal(ScalarValue::Utf8(None));
+    let lit = Expr::Literal(ScalarValue::Utf8(None), None);
     for function in string::functions() {
         // default to 4 args (though some exprs like substr have error checking)
         let num_args = 4;
