@@ -22,13 +22,16 @@ use arrow::array::{Array, AsArray};
 use arrow::datatypes::Fields;
 use arrow::util::display::ArrayFormatter;
 use arrow::{array, array::ArrayRef, datatypes::DataType, record_batch::RecordBatch};
-use datafusion::common::format::DEFAULT_CLI_FORMAT_OPTIONS;
 use datafusion::common::DataFusionError;
+use datafusion::config::ConfigField;
 use std::path::PathBuf;
 use std::sync::LazyLock;
 
 /// Converts `batches` to a result as expected by sqllogictest.
-pub fn convert_batches(batches: Vec<RecordBatch>) -> Result<Vec<Vec<String>>> {
+pub fn convert_batches(
+    batches: Vec<RecordBatch>,
+    is_spark_path: bool,
+) -> Result<Vec<Vec<String>>> {
     if batches.is_empty() {
         Ok(vec![])
     } else {
@@ -46,7 +49,16 @@ pub fn convert_batches(batches: Vec<RecordBatch>) -> Result<Vec<Vec<String>>> {
                 )));
             }
 
-            let new_rows = convert_batch(batch)?
+            // Convert a single batch to a `Vec<Vec<String>>` for comparison, flatten expanded rows, and normalize each.
+            let new_rows = (0..batch.num_rows())
+                .map(|row| {
+                    batch
+                        .columns()
+                        .iter()
+                        .map(|col| cell_to_string(col, row, is_spark_path))
+                        .collect::<Result<Vec<String>>>()
+                })
+                .collect::<Result<Vec<Vec<String>>>>()?
                 .into_iter()
                 .flat_map(expand_row)
                 .map(normalize_paths);
@@ -162,19 +174,6 @@ static WORKSPACE_ROOT: LazyLock<object_store::path::Path> = LazyLock::new(|| {
     object_store::path::Path::parse(sanitized_workplace_root).unwrap()
 });
 
-/// Convert a single batch to a `Vec<Vec<String>>` for comparison
-fn convert_batch(batch: RecordBatch) -> Result<Vec<Vec<String>>> {
-    (0..batch.num_rows())
-        .map(|row| {
-            batch
-                .columns()
-                .iter()
-                .map(|col| cell_to_string(col, row))
-                .collect::<Result<Vec<String>>>()
-        })
-        .collect()
-}
-
 macro_rules! get_row_value {
     ($array_type:ty, $column: ident, $row: ident) => {{
         let array = $column.as_any().downcast_ref::<$array_type>().unwrap();
@@ -193,7 +192,7 @@ macro_rules! get_row_value {
 ///
 /// Floating numbers are rounded to have a consistent representation with the Postgres runner.
 ///
-pub fn cell_to_string(col: &ArrayRef, row: usize) -> Result<String> {
+pub fn cell_to_string(col: &ArrayRef, row: usize, is_spark_path: bool) -> Result<String> {
     if !col.is_valid(row) {
         // represent any null value with the string "NULL"
         Ok(NULL_STR.to_string())
@@ -210,7 +209,12 @@ pub fn cell_to_string(col: &ArrayRef, row: usize) -> Result<String> {
                 Ok(f32_to_str(get_row_value!(array::Float32Array, col, row)))
             }
             DataType::Float64 => {
-                Ok(f64_to_str(get_row_value!(array::Float64Array, col, row)))
+                let result = get_row_value!(array::Float64Array, col, row);
+                if is_spark_path {
+                    Ok(spark_f64_to_str(result))
+                } else {
+                    Ok(f64_to_str(result))
+                }
             }
             DataType::Decimal128(_, scale) => {
                 let value = get_row_value!(array::Decimal128Array, col, row);
@@ -236,12 +240,20 @@ pub fn cell_to_string(col: &ArrayRef, row: usize) -> Result<String> {
             DataType::Dictionary(_, _) => {
                 let dict = col.as_any_dictionary();
                 let key = dict.normalized_keys()[row];
-                Ok(cell_to_string(dict.values(), key)?)
+                Ok(cell_to_string(dict.values(), key, is_spark_path)?)
             }
             _ => {
-                let f =
-                    ArrayFormatter::try_new(col.as_ref(), &DEFAULT_CLI_FORMAT_OPTIONS);
-                Ok(f.unwrap().value(row).to_string())
+                let mut datafusion_format_options =
+                    datafusion::config::FormatOptions::default();
+
+                datafusion_format_options.set("null", "NULL").unwrap();
+
+                let arrow_format_options: arrow::util::display::FormatOptions =
+                    (&datafusion_format_options).try_into().unwrap();
+
+                let f = ArrayFormatter::try_new(col.as_ref(), &arrow_format_options)?;
+
+                Ok(f.value(row).to_string())
             }
         }
         .map_err(DFSqlLogicTestError::Arrow)
@@ -280,7 +292,9 @@ pub fn convert_schema_to_types(columns: &Fields) -> Vec<DFColumnType> {
                 if key_type.is_integer() {
                     // mapping dictionary string types to Text
                     match value_type.as_ref() {
-                        DataType::Utf8 | DataType::LargeUtf8 => DFColumnType::Text,
+                        DataType::Utf8 | DataType::LargeUtf8 | DataType::Utf8View => {
+                            DFColumnType::Text
+                        }
                         _ => DFColumnType::Another,
                     }
                 } else {
