@@ -34,12 +34,15 @@ pub use engines::DFColumnType;
 pub use engines::DFOutput;
 pub use engines::DFSqlLogicTestError;
 pub use engines::DataFusion;
+pub use engines::DataFusionSubstraitRoundTrip;
 
 #[cfg(feature = "postgres")]
 pub use engines::Postgres;
 
+mod filters;
 mod test_context;
 mod util;
 
+pub use filters::*;
 pub use test_context::TestContext;
 pub use util::*;
